package com.keyware.sonar.cxx.rules.checkers;

import com.sonar.cxx.sslr.api.AstNode;
import com.sonar.cxx.sslr.api.Grammar;
import org.sonar.check.Priority;
import org.sonar.check.Rule;
import org.sonar.cxx.parser.CxxGrammarImpl;
import org.sonar.cxx.squidbridge.annotations.ActivatedByDefault;
import org.sonar.cxx.squidbridge.annotations.SqaleConstantRemediation;
import org.sonar.cxx.squidbridge.checks.SquidCheck;

import java.util.ArrayList;
import java.util.List;

@Rule(key = "NumericalCopyChecker", name = "应返回该私有数值的副本", description = "当私有数组成员的数据需要作为公有函数的返回值时，应返回该私有数值的副本", priority = Priority.INFO, tags = {"28suo"})
@ActivatedByDefault
@SqaleConstantRemediation("5min")
public class NumericalCopyChecker extends SquidCheck<Grammar> {//当私有数组成员的数据需要作为公有函数的返回值时，应返回该私有数值的副本。

    public void init() {
        // 订阅要检查AST节点类型，用于在visitNode方法中检查该类型节点
        this.subscribeTo(
                CxxGrammarImpl.functionDefinition
        );
    }
    public void visitNode(AstNode astNode) {
        List<AstNode> astNodeList = astNode.getDescendants(CxxGrammarImpl.declarator);
        for (AstNode as : astNodeList) {
            if (as.getTokenValue().contains("&") || as.getTokenValue().contains("*")) {
                List<String> lists = new ArrayList();
                List<AstNode> descendant = astNode.getDescendants(CxxGrammarImpl.expression);
                for (AstNode desc : descendant) {
                    lists.add(desc.getTokenValue());
                }
                List<AstNode> descendan = astNode.getDescendants(CxxGrammarImpl.statement);
                for (AstNode ast : descendan) {
                    List<AstNode> descendants = ast.getDescendants(CxxGrammarImpl.expression);
                    for (AstNode des : descendants) {
                        if (lists.contains(des.getTokenValue())) {
                            getContext().createLineViolation(this, "应返回该私有数值的副本", des);
                        }
                    }
                }
            }
        }


    }

}


